--TEST--
V8\ObjectValue
--SKIPIF--
<?php if (!extension_loaded("v8")) print "skip"; ?>
--FILE--
<?php

/** @var \Phpv8Testsuite $helper */
$helper = require '.testsuite.php';

require '.v8-helpers.php';
$v8_helper = new PhpV8Helpers($helper);

$isolate = new \V8\Isolate();
$global_template = new V8\ObjectTemplate($isolate);

$context = new V8\Context($isolate, $global_template);

$value = new V8\ObjectValue($context);


$helper->header('Object representation');
$helper->dump($value);
$helper->space();

$helper->assert('ObjectValue extends Value', $value instanceof \V8\Value);
$helper->assert('ObjectValue does not extend PrimitiveValue', !($value instanceof \V8\PrimitiveValue));
$helper->assert('ObjectValue implements AdjustableExternalMemoryInterface', $value instanceof \V8\AdjustableExternalMemoryInterface);
$helper->assert('ObjectValue is instanceof Object', $value->InstanceOf($context, $context->GlobalObject()->Get($context, new \V8\StringValue($isolate, 'Object'))));
$helper->line();

$helper->header('Accessors');
$helper->method_matches($value, 'GetIsolate', $isolate);
$helper->method_matches($value, 'GetContext', $context);
$helper->space();

$helper->header('Getters');
$helper->assert('GetIdentityHash is integer', gettype($value->GetIdentityHash()), 'integer');
$helper->space();

$v8_helper->run_checks($value, 'Checkers');

$helper->header('Converters');
$helper->dump_object_methods($value, ['@@default' => [$context]], new RegexpFilter('/^To/'));

?>
--EXPECT--
Object representation:
----------------------
object(V8\ObjectValue)#6 (2) {
  ["isolate":"V8\Value":private]=>
  object(V8\Isolate)#3 (0) {
  }
  ["context":"V8\ObjectValue":private]=>
  object(V8\Context)#5 (1) {
    ["isolate":"V8\Context":private]=>
    object(V8\Isolate)#3 (0) {
    }
  }
}


ObjectValue extends Value: ok
ObjectValue does not extend PrimitiveValue: ok
ObjectValue implements AdjustableExternalMemoryInterface: ok
ObjectValue is instanceof Object: ok

Accessors:
----------
V8\ObjectValue::GetIsolate() matches expected value
V8\ObjectValue::GetContext() matches expected value


Getters:
--------
GetIdentityHash is integer: ok


Checkers:
---------
V8\ObjectValue(V8\Value)->TypeOf(): V8\StringValue->Value(): string(6) "object"

V8\ObjectValue->IsCallable(): bool(false)
V8\ObjectValue->IsConstructor(): bool(false)
V8\ObjectValue(V8\Value)->IsUndefined(): bool(false)
V8\ObjectValue(V8\Value)->IsNull(): bool(false)
V8\ObjectValue(V8\Value)->IsNullOrUndefined(): bool(false)
V8\ObjectValue(V8\Value)->IsTrue(): bool(false)
V8\ObjectValue(V8\Value)->IsFalse(): bool(false)
V8\ObjectValue(V8\Value)->IsName(): bool(false)
V8\ObjectValue(V8\Value)->IsString(): bool(false)
V8\ObjectValue(V8\Value)->IsSymbol(): bool(false)
V8\ObjectValue(V8\Value)->IsFunction(): bool(false)
V8\ObjectValue(V8\Value)->IsArray(): bool(false)
V8\ObjectValue(V8\Value)->IsObject(): bool(true)
V8\ObjectValue(V8\Value)->IsBoolean(): bool(false)
V8\ObjectValue(V8\Value)->IsNumber(): bool(false)
V8\ObjectValue(V8\Value)->IsInt32(): bool(false)
V8\ObjectValue(V8\Value)->IsUint32(): bool(false)
V8\ObjectValue(V8\Value)->IsDate(): bool(false)
V8\ObjectValue(V8\Value)->IsArgumentsObject(): bool(false)
V8\ObjectValue(V8\Value)->IsBooleanObject(): bool(false)
V8\ObjectValue(V8\Value)->IsNumberObject(): bool(false)
V8\ObjectValue(V8\Value)->IsStringObject(): bool(false)
V8\ObjectValue(V8\Value)->IsSymbolObject(): bool(false)
V8\ObjectValue(V8\Value)->IsNativeError(): bool(false)
V8\ObjectValue(V8\Value)->IsRegExp(): bool(false)
V8\ObjectValue(V8\Value)->IsAsyncFunction(): bool(false)
V8\ObjectValue(V8\Value)->IsGeneratorFunction(): bool(false)
V8\ObjectValue(V8\Value)->IsGeneratorObject(): bool(false)
V8\ObjectValue(V8\Value)->IsPromise(): bool(false)
V8\ObjectValue(V8\Value)->IsMap(): bool(false)
V8\ObjectValue(V8\Value)->IsSet(): bool(false)
V8\ObjectValue(V8\Value)->IsMapIterator(): bool(false)
V8\ObjectValue(V8\Value)->IsSetIterator(): bool(false)
V8\ObjectValue(V8\Value)->IsWeakMap(): bool(false)
V8\ObjectValue(V8\Value)->IsWeakSet(): bool(false)
V8\ObjectValue(V8\Value)->IsArrayBuffer(): bool(false)
V8\ObjectValue(V8\Value)->IsArrayBufferView(): bool(false)
V8\ObjectValue(V8\Value)->IsTypedArray(): bool(false)
V8\ObjectValue(V8\Value)->IsUint8Array(): bool(false)
V8\ObjectValue(V8\Value)->IsUint8ClampedArray(): bool(false)
V8\ObjectValue(V8\Value)->IsInt8Array(): bool(false)
V8\ObjectValue(V8\Value)->IsUint16Array(): bool(false)
V8\ObjectValue(V8\Value)->IsInt16Array(): bool(false)
V8\ObjectValue(V8\Value)->IsUint32Array(): bool(false)
V8\ObjectValue(V8\Value)->IsInt32Array(): bool(false)
V8\ObjectValue(V8\Value)->IsFloat32Array(): bool(false)
V8\ObjectValue(V8\Value)->IsFloat64Array(): bool(false)
V8\ObjectValue(V8\Value)->IsDataView(): bool(false)
V8\ObjectValue(V8\Value)->IsSharedArrayBuffer(): bool(false)
V8\ObjectValue(V8\Value)->IsProxy(): bool(false)


Converters:
-----------
V8\ObjectValue(V8\Value)->ToBoolean():
    object(V8\BooleanValue)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToNumber():
    object(V8\NumberValue)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToString():
    object(V8\StringValue)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToDetailString():
    object(V8\StringValue)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToObject():
    object(V8\ObjectValue)#6 (2) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
      ["context":"V8\ObjectValue":private]=>
      object(V8\Context)#5 (1) {
        ["isolate":"V8\Context":private]=>
        object(V8\Isolate)#3 (0) {
        }
      }
    }
V8\ObjectValue(V8\Value)->ToInteger():
    object(V8\Int32Value)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToUint32():
    object(V8\Int32Value)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToInt32():
    object(V8\Int32Value)#117 (1) {
      ["isolate":"V8\Value":private]=>
      object(V8\Isolate)#3 (0) {
      }
    }
V8\ObjectValue(V8\Value)->ToArrayIndex(): V8\Exceptions\Exception: Failed to convert
